﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;

namespace PTMedicalInsurance.Entity.Local
{
    /// <summary>
    /// 增量查询下载医院项目目录query_yyxm_info_by_sxh的请求参数
    /// </summary>
    public class QueryYyxmInfoBySxhRequest
    {
        /// <summary>
        /// 出参返回格式p_filetype的值为“excel”、“txt”、或“json”其中一种。不传时，默认为“json”。
        /// </summary>
        public string p_filetype { get; set; }

        /// <summary>
        /// 医院端目前本地数据库中目录最大的流水号。长度为20
        /// </summary>
        public double p_sxh { get; set; }
    }

    /// <summary>
    /// 增量查询下载医院项目目录query_yyxm_info_by_sxh的返回结果
    /// </summary>
    public class QueryYyxmInfoBySxhResponse
    {
        /// <summary>
        /// 执行代码
        /// </summary>
        public string resultcode { get; set; }

        /// <summary>
        /// 医院项目信息数据集
        /// </summary>
        public List<YyxmDs> yyxm_ds { get; set; }

        /// <summary>
        /// 是否还有需更新的记录
        /// </summary>
        public string sfjxxz { get; set; }

        /// <summary>
        /// 本次下载的记录条数
        /// </summary>
        public int sl { get; set; }

        /// <summary>
        /// 限价信息数据集
        /// </summary>
        public List<XjDs> xj_ds { get; set; }
    }

    /// <summary>
    /// 医院项目信息类
    /// </summary>
    public class YyxmDs
    {
        /// <summary>
        /// 记录顺序号
        /// </summary>
        public double sxh { get; set; }

        /// <summary>
        /// 医院项目编码
        /// </summary>
        public string yyxmbm { get; set; }

        /// <summary>
        /// 医院项目名称
        /// </summary>
        public string yyxmmc { get; set; }

        /// <summary>
        /// 自负比例的小数，比如自负比例为15%，则该数值为0.15
        /// </summary>
        public double zfbl { get; set; }

        /// <summary>
        /// 自付比例说明
        /// </summary>
        public string sm { get; set; }

        /// <summary>
        /// 医疗项目编码
        /// </summary>
        public string ylxmbm { get; set; }

        /// <summary>
        /// 包装单位
        /// </summary>
        public string gg { get; set; }

        /// <summary>
        /// 单位
        /// </summary>
        public string dw { get; set; }

        /// <summary>
        /// 参考价
        /// </summary>
        public double ckj { get; set; }

        /// <summary>
        /// 具体值调用数据字典接口获取，代码编号：JXMC
        /// </summary>
        public string jxm { get; set; }

        /// <summary>
        /// 生产企业
        /// </summary>
        public string scqy { get; set; }

        /// <summary>
        /// 处方药标志，1:是,0或空:否
        /// </summary>
        public string cfybz { get; set; }

        /// <summary>
        /// GMP标志，1:是,0或空:否
        /// </summary>
        public string gmpbz { get; set; }

        /// <summary>
        /// 最小规格
        /// </summary>
        public string zxgg { get; set; }

        /// <summary>
        /// 包含数量
        /// </summary>
        public double bzsl { get; set; }

        /// <summary>
        /// 更新时间
        /// </summary>
        public string gxsj { get; set; }

        /// <summary>
        /// 自付比例的起始日期
        /// </summary>
        public DateTime qsrq { get; set; }

        /// <summary>
        /// 自付比例的终止日期
        /// </summary>
        public DateTime zzrq { get; set; }

        /// <summary>
        /// 审批标志‘0’：尚未审批；‘1’:审批通过；‘2’：审批未通过, 可调用数据字典接口获取，代码编号：SPBZ
        /// </summary>
        public string spbz { get; set; }

        /// <summary>
        /// 险种标志医疗 C，工伤 D，生育 E，可调用数据字典接口获取，代码编号：XZBZ
        /// </summary>
        public string xzbz { get; set; }

        /// <summary>
        /// 药品标志1：药品, 其他具体值调用数据字典接口获取，代码编号：YPBZ
        /// </summary>
        public string ypbz { get; set; }

        /// <summary>
        /// 原社保机构类型，A：职工，B：居民，其他具体值调用数据字典接口获取，代码编号：RQLB
        /// </summary>
        public string rqlb { get; set; }

        /// <summary>
        /// 济南专用, 补充医疗比例
        /// </summary>
        public double bcylbl { get; set; }

        /// <summary>
        /// 济南专用, 使用人员类别
        /// </summary>
        public string syrylb { get; set; }

        /// <summary>
        /// 包装规格
        /// </summary>
        public string bzgg { get; set; }

        /// <summary>
        /// 自付比例的医疗统筹类别
        /// </summary>
        public string yltclb { get; set; }

        /// <summary>
        /// 单价
        /// </summary>
        public double dj { get; set; }

        /// <summary>
        /// 国标项目编码
        /// </summary>
        public string gbxmbm { get; set; }

        /// <summary>
        /// 国标项目名称
        /// </summary>
        public string gbxmmc { get; set; }
    }

        /// <summary>
        /// 限价信息类
        /// </summary>
        public class XjDs
    {
        /// <summary>
        /// 医疗项目编码
        /// </summary>
        public string ylxmbm { get; set; }

        /// <summary>
        /// 原社保机构类型：A：职工，B：居民，其他具体值调用数据字典接口获取，代码编号：RQLB
        /// </summary>
        public string rqlb { get; set; }

        /// <summary>
        /// 4：门诊大病，5：意外伤害，6：普通门诊统筹，其他具体值调用数据字典接口获取，代码编号：YLTCLB
        /// </summary>
        public string yltclb { get; set; }

        /// <summary>
        /// 待遇人员类别，具体值调用数据字典接口获取，代码编号：DYRYLB
        /// </summary>
        public string dyrylb { get; set; }

        /// <summary>
        /// 自付比例的起始日期
        /// </summary>
        public DateTime qsrq { get; set; }

        /// <summary>
        /// 自付比例的终止日期
        /// </summary>
        public DateTime zzrq { get; set; }

        /// <summary>
        /// 限价
        /// </summary>
        public double xj { get; set; }

        /// <summary>
        /// 备注
        /// </summary>
        public string bz { get; set; }

        /// <summary>
        /// 医院编码
        /// </summary>
        public string yybm { get; set; }

        /// <summary>
        /// 医院级别
        /// </summary>
        public string yyjb { get; set; }

        /// <summary>
        /// 医疗机构性质
        /// </summary>
        public string yljgxz { get; set; }
    }
    }
